#ifndef RASTER_DRAW_H_
#define RASTER_DRAW_H_
#include "clut_t.h"


//
// primitive drawing functions
// and structures
//

namespace raster
 {
  //
  // called before rendering each frame
  //
  // x_res and y_res are used for 2d clipping and setting
  // up an array of pointers to the buffer 
  extern void __fastcall set_surface(int x_res, int y_res, void * buffer, u32 buf_pitch);


   //
   // standard texture primitive
   //
   struct texture
   {
      const byte * map;          // raw data, 8 bit palettized

      byte l2_dim[2];      // dimensions of texture in bits, e.g. 4,5 would be a 16,32 texture
                           // textures must be a power of 2 in both directions.
                           // they can of course be padded if tiling is not needed.
  
      byte shade_bits;     // number of bits used in shading, must be between 0..8
                           // e.g. 5 means 32 shade levels

      byte has_transparency;   // whether or not this texture has transparency, must be 
                               // 1 or 0

      clut_miser::clut_t clut;        // colour lookup table, generated clut_miser and palette
      int   id;                       // unique identifier

   };

   //
   // note: this must be called before using a texture to draw with
   //       this currently sets the id of the texture

   extern void register_texture( texture * tex );
   

   struct solid_surface_desc
   {
      byte shade_bits;     // number of bits used in shading, must be between 0..8
      byte padd[3];           // e.g. 5 means 32 shade levels

      clut_miser::clut_t clut;        // colour lookup table, generated by clut_miser 
                          // note: this must be aligned on a (bytes_per_pixel<<shade_bits)
                          // boundary.  for examples, int 16 bit mode with 32 shade levels
                          // ( (int)clut & 63) == 0 
                          // cluts can be stacked on top of each other with no wasted space
   };


   //
   // Notes on data members
   //

   //
   // x,y coordinates:
   //
   // x and y are stored floating point
   // with 0,0 being the top left corner
   // the location of pixels is the actual screen coordinate  of the pixel. 
   // So for a quad to cover pixel 0,0, its corner could be located at:
   //  x0 = -1+eps .. -eps
   //  y0 = -1+eps .. -eps
   //  x1 =  eps .. 1-eps
   //  y1 =  eps .. 1-eps
   //  internally, this is more efficient than having the coordinates of pixels
   //  being halfway between screen coordinates, e.g. pixel 0,0 is at 0.5, 0.5
   //  to fill a 640,480 screen with pixels, your best bet is to have the corners
   //  at -0.5, -0.5,   639.5, 479.5
   //     -0.1, -0.1, 639.1, 479.1, will have the exact same effect (assuming no rounding errors)
   //  if you are used to working with pixels-between-screen coordinates, you can
   //  subtract -0.5 from your camera to screen 2d offset  

   //
   // Shades, us,vs and z_invs
   // the polys store shades,u's,v's and z's for the first 3 vertices
   // passed in.  The parameters are assumed to be linear across the entire
   // poly (even if it has more than 3 verts)
   // note: the area covered by a triangle formed from the first 3 verts must be at least
   // 1/8 pixels (to avoid numerical unstability) this might be a problem for
   // certain clippers that don't handle point-on-plane cases properly


   //
   // Shades: 
   //
   // shades are all in (0.. 1) range 
   //  entries between 0..0.75   are used for diffuse shading,
   //  entries between 0.75..1.0 have specular components added in
   //

   //
   // u,v:
   //
   // u,vs are in texture widths and heights, repectively.
   // e.g. u == 0.5 in a texture with width 64 is 32.0 in texels
   //

   //
   // z_invs:
   //   1.0/z at each vertex.  the actual scale doesn't matter, all that matters is the 
   // relative values between vertices. 
   //  e.g. passing in 1, 1, 1 would give you an affine mapping.
   // all z_invs for a poly must be positive.
   //
   //


   //
   // standard screen space vertice
   //

   struct vert_2d
   {
    float x, y;    //  0,0 is at top left corner

    vert_2d * next;  // circular linked list
   };



   //
   // perspectively correct linear shaded  textured
   //
   struct poly_plt 
   {
    float shades[3];
    float us[3];
    float vs[3];
 
    float z_invs[3];  // 1.0/z 

    const vert_2d * verts; // pointer to any vert in list

    const texture * tmap;
   };

   //
   // perspectively correct flat shaded  textured
   //
   struct poly_pft 
   {
    float shade;
    float us[3];
    float vs[3];
 
    float z_invs[3];  // 1.0/z 

    const vert_2d * verts; // pointer to any vert in list

    const texture * tmap;
   };

   //
   // perspectively correct linear shaded solid
   //
   struct poly_pls 
   {
    float shades[3];
 
    float z_invs[3];  // 1.0/z 

    const vert_2d * verts; // pointer to any vert in list

    const solid_surface_desc * surf;  // surface color info
   };


   //
   // afine linear shaded textured
   //
   struct poly_alt 
   {
    float shades[3];
    float us[3];
    float vs[3];

    const vert_2d * verts; // pointer to any vert in list

    const texture * tmap;
   };


   //
   // afine flat shaded textured
   //
   struct poly_aft 
   {
    float shade;
    float us[3];
    float vs[3];

    const vert_2d * verts; // pointer to any vert in list

    const texture * tmap;
   };

   //
   // afine linear shaded solid
   //
   struct poly_als 
   {
    float shades[3];

    const vert_2d * verts; // pointer to any vert in list

    const solid_surface_desc * surf;  // surface color info
   };

   //
   // afine flat shaded solid
   //
   struct poly_afs 
   {
    float shade;

    const vert_2d * verts; // pointer to any vert in list

    const solid_surface_desc * surf;  // surface color info
    int translucent; // flag
   };


   //
   // Polygon outline drawer.  draws edges of polys.
   //
   struct poly_outline 
   {
    float thickness; // how wide to draw the edges, in pixels

    float shade;                // shade and surf determine the color of the lines
 
    const vert_2d * verts; // pointer to any vert in list

    const solid_surface_desc * surf;  // surface color info
   };


 // draw poly funcs
  extern void __fastcall draw_poly(const poly_plt * p);
  extern void __fastcall draw_poly(const poly_pft * p);
  extern void __fastcall draw_poly(const poly_pls * p);

  extern void __fastcall draw_poly(const poly_alt * p);
  extern void __fastcall draw_poly(const poly_aft * p);

  extern void __fastcall draw_poly(const poly_als * p);
  extern void __fastcall draw_poly(const poly_afs * p);
  
  extern void __fastcall draw_poly(const poly_outline * p);
  
};




#endif